import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, max, format, extent } from 'd3';
import { useData } from './useData';
import { AxisLeft } from './AxisLeft';
import { AxisBottom } from './AxisBottom';
import { Marks } from './Marks';

const width = window.innerWidth;
const height = window.innerHeight;
const margin = {
  top: 20,
  right: 30,
  bottom: 65,
  left: 90,
};

const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;

const xValue = (d) => d.petal_length;
const xAxisLabel = 'Petal Length';

const yValue = (d) => d.sepal_width;
const yAxisLabel = 'Sepal Width';

const siFormat = format('.2s');
const yAxisTickFormat = (tickValue) =>
  siFormat(tickValue).replace('G', 'B');

const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisLeft
          yScale={yScale}
          tickFormat={yAxisTickFormat}
          innerWidth={innerWidth}
          tickOffset={5}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          x={-yAxisLabelOffset}
          y={innerHeight / 2}
          style={{transformOrigin: `${-yAxisLabelOffset}px ${innerHeight/2}px`}}
          transform="rotate(-90)"
        >
          {yAxisLabel}
        </text>

        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickOffset={5}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>

        <Marks
          data={data}
          xScale={xScale}
          yScale={yScale}
          innerHeight={innerHeight}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={yAxisTickFormat}
          circleRadius={7}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
